// **********************************************************************
//
// Copyright (c) 1998
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// Copyright (c) 1998
// Object-Oriented Concepts GmbH
// Ettlingen, Germany
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_NAMED_VALUE_H
#define OB_NAMED_VALUE_H

namespace CORBA
{

class NamedValue;
typedef NamedValue* NamedValue_ptr;
typedef NamedValue_ptr NamedValueRef;

class NVList;
typedef NVList* NVList_ptr;
typedef NVList_ptr NVListRef;

} // End of namespace CORBA

void OBDuplicate(CORBA::NamedValue_ptr);
void OBRelease(CORBA::NamedValue_ptr);
void OBDuplicate(CORBA::NVList_ptr);
void OBRelease(CORBA::NVList_ptr);

namespace CORBA
{

//
// The NamedValue class
//
typedef OB::ObjVar< NamedValue > NamedValue_var;
typedef OB::ObjOut< NamedValue > NamedValue_out;

class NamedValue : public OB::RefCount
{
    String_var name_;
    mutable Any_var value_;
    Flags flags_;

    //
    // Hide copy-constructor and assignment operator
    //
    NamedValue(const NamedValue&);
    void operator=(const NamedValue&);

protected:

    NamedValue();
    NamedValue(char*, Any*, Flags);

    friend class NVList;  // NVLists may create NamedValues
    friend class Request; // Requests may create NamedValues
    friend class ORB;     // ORBs may create NamedValues

public:

    static inline NamedValue_ptr _duplicate(NamedValue_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline NamedValue_ptr _nil()
    { return 0; }

    const char* name() const;
    Any* value() const;
    Flags flags() const;
};

inline void
release(NamedValue_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

inline Boolean
is_nil(NamedValue_ptr p)
{
    return p == 0;
}

//
// The NVList class
//
typedef OB::ObjVar< NVList > NVList_var;
typedef OB::ObjOut< NVList > NVList_out;

class NVList : public OB::RefCount
{
    OB::ObjSeq< NamedValue, int > namedValueSeq_;
    
    //
    // Hide copy-constructor and assignment operator
    //
    NVList(const NVList&);
    void operator=(const NVList&);

protected:

    NVList() { };
    NVList(ULong);

    friend class Context; // Contexts may create NVLists
    friend class Request; // Requests may create NVLists
    friend class ORB;     // ORBs create NVLists

public:

    static inline NVList_ptr _duplicate(NVList_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline NVList_ptr _nil()
    { return 0; }

    ULong count() const;

    NamedValue_ptr add(Flags);
    NamedValue_ptr add_item(const char*, Flags);
    NamedValue_ptr add_value(const char*, const Any&, Flags);

    NamedValue_ptr add_item_consume(char*, Flags);
    NamedValue_ptr add_value_consume(char*, Any*, Flags);

    NamedValue_ptr item(ULong);

    void remove(ULong);
};

inline void
release(NVList_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

inline Boolean
is_nil(NVList_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

#endif
